//! Types for 2D mathematics.

pub struct PixelSpace;
pub type PixelPoint = euclid::Point2D<i32, PixelSpace>;
pub type PixelSize = euclid::Size2D<i32, PixelSpace>;
pub type PixelRect = euclid::Rect<i32, PixelSpace>;
pub type PixelBox = euclid::Box2D<i32, PixelSpace>;
pub type PixelVector = euclid::Vector2D<i32, PixelSpace>;
pub type PixelSideOffsets = euclid::SideOffsets2D<i32, PixelSpace>;

pub fn clip_to(
    img1_clip_box: PixelBox,
    img2_clip_box: PixelBox,
    img1_victim_box: PixelBox,
    img2_victim_box: PixelBox,
) -> Option<(PixelBox, PixelBox)> {
    if img1_victim_box.size() != img2_victim_box.size() {
        return None;
    }

    let adj = PixelSideOffsets {
        left: (img1_clip_box.min.x - img1_victim_box.min.x)
            .max(img2_clip_box.min.x - img2_victim_box.min.x)
            .max(0),
        top: (img1_clip_box.min.y - img1_victim_box.min.y)
            .max(img2_clip_box.min.y - img2_victim_box.min.y)
            .max(0),
        right: (img1_victim_box.max.x - img1_clip_box.max.x)
            .max(img2_victim_box.max.x - img2_clip_box.max.x)
            .max(0),
        bottom: (img1_victim_box.max.y - img1_clip_box.max.y)
            .max(img2_victim_box.max.y - img2_clip_box.max.y)
            .max(0),
        ..Default::default()
    };

    Some((
        img1_victim_box.inner_box(adj),
        img2_victim_box.inner_box(adj),
    ))
}

/*pub fn clip_to(
    img1_size: PixelSize,
    pt1: PixelPoint,
    img2_size: PixelSize,
    pt2: PixelPoint,
    sz: PixelSize,
) -> Option<(PixelBox, PixelBox)> {
    let left = (-pt1.x).max(-pt2.x).max(0);
    let top = (-pt1.y).max(-pt2.y).max(0);
    let right = ((sz.width + pt1.x) - img1_size.width as i32)
        .max((sz.width + pt2.x) - img2_size.width as i32)
        .max(0);
    let bottom = ((sz.height + pt1.y) - img1_size.height as i32)
        .max((sz.height + pt2.y) - img2_size.height as i32)
        .max(0);

    let adj = PixelSideOffsets {
        left,
        right,
        top,
        bottom,
        ..Default::default()
    };

    Some((
        PixelBox::from_origin_and_size(pt1, sz).inner_box(adj),
        PixelBox::from_origin_and_size(pt2, sz).inner_box(adj),
    ))
}*/
