use crate::{
    bitmap::{BitmapDataMut, MakeMutView},
    colour::Colour,
    formats::PixelFormatSpec,
    math::PixelBox,
};

use super::{
    dispatch::{GenericUnary, UnaryDispatch},
    KahloOps, UnaryOpSpec,
};

pub struct FillRegionOp;

#[cfg(test)]
const _: () = {
    use super::generate;
    impl generate::UnaryInputSpec for FillRegionOp {
        fn gen<'l>() -> impl generate::ValueGeneratorList<'l, OutputTuple = Self::Params<'l>> {
            (generate::dst_area, generate::random_colour)
        }
    }
};

impl GenericUnary<(PixelBox, Colour)> for FillRegionOp {
    fn perform<Format: PixelFormatSpec>(
        mut write_data: &mut dyn BitmapDataMut,
        params: &(PixelBox, Colour),
    ) {
        // make a view and forward to FillOp
        let mut subregion = write_data.make_view_mut::<Format>(params.0);
        subregion.fill(params.1);
    }
}

impl UnaryOpSpec for FillRegionOp {
    type Params<'l> = (PixelBox, Colour);
    fn build<'l>() -> UnaryDispatch<Self::Params<'l>> {
        UnaryDispatch::from_generic::<Self>()
    }
}
