use crate::math;

pub fn clip_to(
    img1_size: math::PixelSize,
    pt1: math::PixelPoint,
    img2_size: math::PixelSize,
    pt2: math::PixelPoint,
    sz: math::PixelSize,
) -> Option<(math::PixelBox, math::PixelBox)> {
    let left = (-pt1.x).max(-pt2.x).max(0);
    let top = (-pt1.y).max(-pt2.y).max(0);
    let right = ((sz.width + pt1.x) - img1_size.width as i32)
        .max((sz.width + pt2.x) - img2_size.width as i32)
        .max(0);
    let bottom = ((sz.height + pt1.y) - img1_size.height as i32)
        .max((sz.height + pt2.y) - img2_size.height as i32)
        .max(0);

    let adj = math::PixelSideOffsets {
        left,
        right,
        top,
        bottom,
        ..Default::default()
    };

    Some((
        math::PixelBox::from_origin_and_size(pt1, sz).inner_box(adj),
        math::PixelBox::from_origin_and_size(pt2, sz).inner_box(adj),
    ))
}
