use super::{Readable, Writable};
use crate::{colour::Colour, formats::*, prelude::*};

impl<T: BitmapAccess<A8>> Readable<A8> for T {
    type ReadResult = u8;
    fn get_pixel(&self, x: usize, y: usize) -> u8 {
        self.data()[x + (y * self.row_stride())]
    }
}

impl<T: BitmapMutAccess<A8>> Writable<A8> for T {
    fn set_pixel(&mut self, x: usize, y: usize, to: Self::ReadResult) {
        let index = x + (y * self.row_stride());
        self.data_mut()[index] = to;
    }
}

impl<T: BitmapAccess<Rgba32>> Readable<Rgba32> for T {
    type ReadResult = Colour;
    fn get_pixel(&self, x: usize, y: usize) -> Colour {
        let index = (x * Rgba32::PIXEL_WIDTH) + (y * self.row_stride());
        Colour::from_le_bytes(&self.data()[index..(index + 4)])
    }
}

impl<T: BitmapMutAccess<Rgba32>> Writable<Rgba32> for T {
    fn set_pixel(&mut self, x: usize, y: usize, to: Self::ReadResult) {
        let index = x * Rgba32::PIXEL_WIDTH + (y * self.row_stride());
        self.data_mut()[index..(index + 4)].copy_from_slice(&to.as_le_bytes());
    }
}
